Package: multitaper
Version: 1.0-17
Title: Spectral Analysis Tools using the Multitaper Method
Author: Karim Rahim <karim.rahim@queensu.ca>, Wesley S. Burr <wesleyburr@trentu.ca>
Maintainer: Karim Rahim <karim.rahim@queensu.ca>
Depends: R (>= 3.0), methods
Suggests: psd, fftwtools, slp
Description: Implements multitaper spectral analysis using discrete prolate spheroidal sequences (Slepians) and sine tapers. It includes an adaptive weighted multitaper spectral estimate, a coherence estimate, Thomson's Harmonic F-test, and complex demodulation. The Slepians sequences are generated efficiently using a tridiagonal matrix solution, and jackknifed confidence intervals are available for most estimates. This package is an implementation of the method described in D.J. Thomson (1982) "Spectrum estimation and harmonic analysis" <doi:10.1109/PROC.1982.12433>.
License: GPL (>= 2)
ByteCompile: true
LazyData: true
URL: https://github.com/krahim/multitaper/
NeedsCompilation: yes
Packaged: 2023-07-20 11:37:23 UTC; karim
Repository: CRAN
Date/Publication: 2023-07-20 12:00:02 UTC
Built: R 4.6.0; aarch64-apple-darwin23; 2026-04-25 16:08:22 UTC; unix
Archs: multitaper.so.dSYM
